#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double Func(double x){
	return (x*x*x*x*x - 5.2*x*x*x + 5.5*x*x - 7*x)*cos(2.0*x); 
}

double ModifiedFunc(double x){
	return (x*x*x*x*x - 5.2 * fabs(x*x*x) + 5.5*x*x - 7*x)*cos(2.0*x); 
}

double rado_2(double a, double b, double (*func)(double x), double fb){
	double x1 = (2*a+b)/3;
	return 3*(b-a)*func(x1)/4+(b-a)*fb/4;	
}

double rado_3(double a, double b, double (*func)(double x), double fb){
	double x1 = ((6+sqrt(6))*a + (4-sqrt(6))*b)/10;
    double x2 = ((6-sqrt(6))*a + (4+sqrt(6))*b)/10;
    double A1 = (16-sqrt(6));
    double A2 = (16+sqrt(6));
    double A3 = 4;
    return (b-a)*(A1*func(x1) + A2*func(x2) + A3*fb)/36;    
}

double Integral(double eps, double a, double b, int *k, int *n, double *richardson , double (*func)(double x)){	
	int i;
	double left, right, step, fright;
	double R2, R3;
	
	(*k) = 1;
	(*n) = 1;
	fright = func(b);
	
	R2 = rado_2(a,b,func,fright);
	R3 = rado_3(a,b,func,fright);
	(*richardson) = R3;		
	
	while (fabs(R2 - R3) >= eps){
		(*k) = (*k)+1;	
		(*richardson) = R3;	
		R2 = 0.0;
		R3 = 0.0;
		(*n) = (*n)*2;
		step = (b-a)/(*n);
		right = a;
		
		for (i=0;i<(*n);i++){
			left = right;
			right = left + step;
			fright = func(right);
			
			R2 += rado_2(left, right, func, fright);	
			R3 += rado_3(left, right, func, fright);		     	
		}		
	}
	(*richardson) = (*richardson) + ((*richardson) - R3)/15;
	return R3;
}

int main(){
	FILE *f = fopen("data.txt", "w");
    if (f == NULL) {
        printf("Error opening file!\n");
        return 1;
    }
    
    int i,j, k, n = 1;
    double Iexact = 2.780135189;
    double eps = 0.1;
    double result;
    double a = -3.0;
    double b = 3.0;
    int n_half;
    double left, right, fright, result_half, richardson, step;
    
    
    for(i = 0; i<9; i++){
    	result = Integral(eps, a, b, &k, &n, &richardson, Func); 	
        fprintf(f, "%.15f %d %.15f %d %.15f\n", fabs(Iexact - result), k, log2((double)(b-a)/n), n, fabs(Iexact - richardson)); 
		eps = eps*0.1;  
	}    
         
    fclose(f);
    return 0;    
}
	