#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define N 5

void SystemFunc(double y[], double f[]) {
	int i, j;
    double A[N][N] = {
        {-0.1, 0.2, 0, 0, 0},
        { 0, -0.2, 0.2, 0, 0},
        { 0, 0, -0.3, 0.2, 0},
        { 0, 0, 0, -0.4, 0.2},
        { 0.2, 0, 0, 0, -0.5}
    };
    for (i = 0; i < N; i++) {
        f[i] = 0;
        for (j = 0; j < N; j++) {
            f[i] += A[i][j] * y[j];
        }
    }
}

void kuttaMersonSystem(double a, double y_0[], double b, double h, double epsilon) {
    double k1[N], k2[N], k3[N], k4[N], k5[N];
    double x = a, R;
    double y[N], f[N], y_temp[N];
    int i;

    FILE *f1 = fopen("data.txt", "w");
    if (f1 == NULL) {
        printf("Error opening file!\n");
        return 1;
    }

    for (i = 0; i < N; i++) {
        y[i] = y_0[i];
    }

    while (x <= b) {  
	    do {
	    	SystemFunc(y, k1);
        
			for (i = 0; i < N; i++) {
			    y_temp[i] = y[i] + h * k1[i] / 3;
			}
	        SystemFunc(y_temp, k2);
	        
			for (i = 0; i < N; i++) {
			    y_temp[i] = y[i] + h*k1[i]/6 + h*k2[i]/6;
			}
	        SystemFunc(y_temp, k3);
	        
			for (i = 0; i < N; i++) {
			    y_temp[i] = y[i] + + h*k1[i]/8 + 3*h*k2[i]/8;
			}
	        SystemFunc(y_temp, k4);
	        
			for (i = 0; i < N; i++) {
			    y_temp[i] = y[i] + h*k1[i]/2 - 3*h*k3[i]/2 +2*h*k4[i];
			}
			SystemFunc(y_temp, k5);
			
			R = 0.0;
			for (i = 0; i < N; i++){
				R += (2*k1[i] - 9*k3[i] + 8*k4[i] - k5[i])*(2*k1[i] - 9*k3[i] + 8*k4[i] - k5[i]);
			}
			R = sqrt(R)*h/30; 
            if (R > epsilon) {
                    h /= 2;
                }
        } while (R > epsilon);

        x += h;
        fprintf(f1, "%.15f", x);
        for (i = 0; i < N; i++) {
            y[i] += (k1[i] + 4*k4[i] + k5[i]) * h / 6;
            fprintf(f1, " %.15f", y[i]);
        }
        fprintf(f1, "\n");
    }
    fclose(f1);
    return;
}

int main() {
    double a = 0, b = 1.5;
    double y_0[N];
    int i;
    double eps = 0.01;
    srand((unsigned int)time(NULL));
    
    for (i = 0; i < N; i++) {
        y_0[i] = (double)rand() / RAND_MAX;
    }

    double h = 0.375; 
    kuttaMersonSystem(a, y_0, b, h, eps);

    return 0;
}

