#include <stdio.h>
#include <stdlib.h>
#include <math.h>
//#include <ctime>

double Func(double x, double y){
	return 1/cos(x) - y*tan(x); 
} 

double Exact(double x){
	return sin(x) + cos(x); 
}

double kuttaMerson(double x0, double y0, double *k, double y[4], double epsilon) {
	int i = 0;
	y[0] = y0;
	double k1, k2, k3, k4, k5, h;
    while (i<3){
	    h = *k;	
        k1 = Func(x0, y0);
		k2 = Func(x0 + h/3, y0 + h*k1/3);
		k3 = Func(x0 + h/3, y0 + h*k1/6 + h*k2/6);
		k4 = Func(x0 + h/2, y0 + h*k1/8 + 3*h*k2/8);
		k5 = Func(x0 + h, y0 + h*k1/2 - 3*h*k3/2 +2*h*k4);
		
		if(fabs(2*k1 - 9*k3 + 8*k4 - k5)*h / 30 > epsilon){
			(*k) /=2;
			continue;
		}		
        y0 += (k1 + 4 * k4 + k5)* h / 6;
        x0 += h;       
        y[++i] = y0;
    }
    return x0;
}

double AdamsBashforth4(double a, double y0, double b, double epsilon, double *k) {
    double y[4]; 
    double fxy[4];
    int i = 0, j, n = 0;
    double h, y_temp, x;
    
    h = (b-a)/4;
    
    x = kuttaMerson(a,y0,&h,y,epsilon);
    for(i = 0; i<4 ; i++){
    	fxy[i] = Func(x-(3-i)*h, y[i]);
	}
    
	y_temp = y[3];	
    while (x + h <= b){
     	y_temp = y[3] + (55*fxy[3] - 59*fxy[2] + 37*fxy[1] - 9*fxy[0]) * h /24;
    	x +=h; 

    	if (fabs(y_temp - y[3])/15> epsilon){
    		h /=2;
		}
		for (j = 0; j < 3; j++) {
			y[j] = y[j+1];
            fxy[j] = fxy[j+1];
        }
        y[3] = y_temp;
        fxy[3] = Func(x, y_temp);  
	}
	(*k) = h; 
	return y_temp;
}


int main() {
    FILE *f = fopen("data.txt", "w");
    if (f == NULL) {
        printf("Error opening file!\n");
        return 1;
    }
    
    int i, j, n;
    double eps = 0.1;
    double error, h;
    double a = 0;
    double b = 1.5;
    double y0 = 1.0;  
    
    for(i = 0; i<9; i++){
    	error = AdamsBashforth4(a, y0, b, eps, &h);
        fprintf(f, "%.15f %.15f %.15f\n", fabs(error - Exact(b)), eps, h);
		eps = eps*0.1; 
	}  
	  
	
	/*double y0_new;
	eps = 0.001;
    srand(time(NULL));
    for (i = 1; i< 6; i++){
    	for(j = 0; j<20; j++){
    		y0_new = ((double)rand() / RAND_MAX * 0.01*i+1)*y0;
    	    error = AdamsBashforth4(a, y0_new, b, eps, &h);
    	    fprintf(f, "%.15f ", error);
	    }
		fprintf(f,"\n");  
	}    */  
         
    fclose(f);
    return 0;    
}
