#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<conio.h>

double *EquallySpaced(double a, double b, int node){   
    int i;
    double step;
    step = (b-a)/(node-1);
    double* x = (double*)malloc(node*sizeof(double));
    for (i=0; i<node; i++){
        x[i] = a + i*step;
    } 
    return x;
}

double *Chebyshev(double a, double b, int node){
    int i;
    double *x= (double*)malloc(node*sizeof(double));
    for (i = 0; i<node; i++){
        x[i] = (a+b)*0.5+ cos(M_PI*(2*i+1)/(2*node))*(b-a)*0.5;
    }
    return x;
}
double *Refine(double a, double b, int node){
    int i;
    double *x= (double*)malloc(node*sizeof(double));
    for (i = 0; i<node ; i++){
        x[i] = a+ 1.0*(b-a)*i*i/(node*node);
    }
    return x;
}

double NewtonBack(double x, double *xk, double *yk, int node){
    double *divDiff = (double*)malloc(node * sizeof(double));
    int i, j;
    double P, t;

    for (i=0;i<node;i++){
        divDiff[i] = yk[i];
    }
    
    for (i = 1;i<node;i++){
        for (j = 0; j<node-i;j++){
            divDiff[j] = (divDiff[j+1]-divDiff[j])/ (xk[i+j]-xk[j]);
        }
    }

    P=0;
    for(i=node-1; i>=0;i--){
        t = 1;
        for (j = node-1; j>i ;j--){
            t = t*(x - xk[j]);
        }
        P+=divDiff[i]*t;

    }  
    return P;
}

double Func1(double x) {
    return (exp(x) - exp(-x))/(exp(x)+exp(-x));
}
double Func2(double x) {
    double sign_x;
    if (x>0) sign_x =1;
    else if (x==0) sign_x =0;
    else sign_x = -1;

    return 3 * sign_x * pow(x*1.0, 4) + 4 * pow(x*1.0, 3) - 12 * pow(x*1.0, 2) + 4;
}
int main(){
    double a = -3.0;
    double b = 3.0;
    int i,j, node;
    int nodes[] = {5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85, 90, 95, 100}; 
    int size = sizeof(nodes) / sizeof(nodes[0]); 
    double *xk1, *yk1, *xk2, *yk2, *xk3, *yk3;
    double error1, error2, error3, t, x0;
    FILE *f;

    f = fopen("data.txt", "w");
    if (f == NULL) {
        printf("Error opening file!\n");
        exit(1);
    }

    for (i = 0; i < size; i++) {
        node = nodes[i];
        
        xk1 = EquallySpaced(a, b, node);
        yk1 = (double*)malloc(node * sizeof(double));
        xk2 = Chebyshev(a, b, node);
        yk2 = (double*)malloc(node * sizeof(double));
        xk3 = Refine(a, b, node);
        yk3 = (double*)malloc(node * sizeof(double));

        error1 = 0;
        error2 = 0;
        error3 = 0;
        
        for (j = 0; j < node; j++) {
            yk1[j] = Func1(xk1[j]);
            yk2[j] = Func1(xk2[j]);
            yk3[j] = Func1(xk3[j]);
        }
        
        for (j = 0; j < 100; j++) {
            x0 = a + j*(b-a)*0.01;
            t = fabs(Func1(x0) - NewtonBack(x0, xk1 ,yk1,node));
            if (t>error1)  error1 = t;
            t = fabs(Func1(x0) - NewtonBack(x0, xk2 ,yk2,node));
            if (t>error2)  error2 = t;
            t = fabs(Func1(x0) - NewtonBack(x0, xk3 ,yk3,node));
            if (t>error3)  error3 = t;
        }
        fprintf(f,"%d %f %f %f\n", node, error1, error2, error3);
    
       free(xk1);
       free(yk1);
       free(xk2);
       free(yk2);
       free(xk3);
       free(yk3);
    }
    fclose(f);
    return 0;
}
